import { isNull, isUndefined } from 'lodash';
import { useEffect, useMemo, useState } from 'react';
import { LocalStorageService } from './localStorage';
import { STRING_PARSER } from './parser';
const updates = new Map();
export function useStorage(key, defaultValue, parser = 'plain') {
    const { SERVICE, PARSER } = useStorage;
    const { serializer, deserializer } = isUndefined(parser) ? PARSER.plain : PARSER[parser];
    const res = useMemo(() => {
        const updateKey = (ov) => {
            const cbs = updates.get(key);
            if (cbs) {
                for (const cb of cbs) {
                    cb(ov);
                }
            }
        };
        return {
            get value() {
                const originValue = SERVICE.getItem(key);
                if (isNull(originValue)) {
                    return defaultValue !== null && defaultValue !== void 0 ? defaultValue : null;
                }
                return deserializer(originValue);
            },
            set: (value) => {
                const originValue = serializer(value);
                SERVICE.setItem(key, originValue);
                updateKey(originValue);
            },
            remove: () => {
                SERVICE.removeItem(key);
                updateKey(null);
            },
        };
    }, [SERVICE, defaultValue, deserializer, key, serializer]);
    const [, setOriginValue] = useState(SERVICE.getItem(key));
    const updatesOfKey = updates.get(key);
    if (isUndefined(updatesOfKey)) {
        updates.set(key, new Set([setOriginValue]));
    }
    else if (!updatesOfKey.has(setOriginValue)) {
        updatesOfKey.add(setOriginValue);
    }
    useEffect(() => {
        var _a;
        (_a = updates.get(key)) === null || _a === void 0 ? void 0 : _a.delete(setOriginValue);
    }, [key]);
    return res;
}
useStorage.SERVICE = new LocalStorageService();
useStorage.PARSER = STRING_PARSER;
useStorage.clear = () => {
    useStorage.SERVICE.clear();
    for (const [, cbs] of updates) {
        for (const cb of cbs) {
            cb(null);
        }
    }
};
